<?php

namespace Ulmus\Attribute\Property;

use Ulmus\Attribute\Attribute;

#[\Attribute(\Attribute::TARGET_PROPERTY)]
class Relation {
    public function __construct(
        public string $type,
        public \Stringable|string|array $key = "",
        public null|\Closure $generateKey = null,
        public null|\Stringable|string|array $foreignKey = null,
        public null|\Stringable|string|array $foreignField = null,
        public array $foreignKeys = [],
        public null|string $bridge = null,
        public null|\Stringable|string|array $bridgeKey = null,
        public null|\Stringable|string|array $bridgeField = null,
        public null|\Stringable|string|array $bridgeForeignKey = null,
        public null|\Stringable|string|array $field = null,
        public null|string $entity = null,
        public null|string $join = null,
        public string $function = "loadAll",
    ) {
        $this->key = Attribute::handleArrayField($this->key);
        $this->foreignKey = Attribute::handleArrayField($this->foreignKey);
        $this->foreignField = Attribute::handleArrayField($this->foreignField);
        $this->bridgeKey = Attribute::handleArrayField($this->bridgeKey);
        $this->bridgeField = Attribute::handleArrayField($this->bridgeField);
        $this->bridgeForeignKey = Attribute::handleArrayField($this->bridgeForeignKey);
        $this->field = Attribute::handleArrayField($this->field);

    }

    public function entity() {
        try {
            $e = $this->entity;
        } catch (\Throwable $ex) {
            throw new \Exception("Your @Relation annotation seems to be missing an `entity` entry.");
        }

        return new $e();
    }

    public function bridge() {
        $e = $this->bridge;

        return new $e();
    }

    public function normalizeType() : string
    {
        return strtolower(str_replace(['-', '_', ' '], '', $this->type));
    }

    public function isOneToOne() : bool
    {
        return $this->normalizeType() === 'onetoone';
    }

    public function isOneToMany() : bool
    {
        return $this->normalizeType() === 'onetomany';
    }

    public function isManyToMany() : bool
    {
        return $this->normalizeType() === 'manytomany';
    }

    public function function() : string
    {
        if ($this->function) {
            return $this->function;
        }
        elseif ($this->isOneToOne()) {
            return 'load';
        }

        return 'loadAll';
    }

    public function hasBridge() : bool
    {
        return (bool) $this->bridge;
    }
}
